@@ -3,6 +3,10 @@ package DhMakePerl::Command::Packaging;
 use strict;
 use warnings;
 
+our $VERSION = '0.77';
+
+use feature 'switch';
+
 =head1 NAME
 
 DhMakePerl::Command::Packaging - common routines for 'make' and 'refresh' dh-make-perl commands
@@ -13,6 +17,7 @@ use base 'DhMakePerl';
 
 __PACKAGE__->mk_accessors(
     qw( start_dir main_dir debian_dir
+        mod_cpan_version
         meta perlname author
         version rules docs examples copyright
         control
@@ -22,6 +27,7 @@ __PACKAGE__->mk_accessors(
 use Array::Unique;
 use Carp qw(confess);
 use CPAN ();
+use CPAN::Meta;
 use Cwd qw( getcwd );
 use Debian::Control::FromCPAN;
 use Debian::Dependencies;
@@ -32,10 +38,11 @@ use File::Find qw(find);
 use File::Path ();
 use File::Spec::Functions qw(catfile catpath splitpath);
 use Parse::DebianChangelog;
+use Text::Balanced qw(extract_quotelike);
 use Text::Wrap qw(fill);
 use User::pwent;
 
-use constant debstdversion => '3.8.4';
+use constant debstdversion => '3.9.5';
 
 our %DEFAULTS = (
     start_dir => getcwd(),
@@ -87,7 +94,7 @@ sub main_file {
 
 =item debian_file(file_name)
 
-Constructs a file name relative to the debian/ subdurectory of the main source
+Constructs a file name relative to the debian/ sub-directory of the main source
 directory.
 
 =cut
@@ -98,6 +105,12 @@ sub debian_file {
     catfile( $self->main_file('debian'), $file );
 }
 
+sub build_pl {
+    my ($self) = @_;
+
+    return $self->main_file('Build.PL');
+}
+
 sub makefile_pl {
     my ($self) = @_;
 
@@ -158,56 +171,49 @@ sub fill_maintainer {
 sub process_meta {
     my ($self) = @_;
 
-    my $file = $self->main_file('META.yml');
-
-    # META.yml non-existent?
-    unless ( -f $file ) {
-        $self->meta({});
-        return;
-    }
+    $self->meta({});
 
     # Command line option nometa causes this function not to be run
     if( $self->cfg->nometa ) {
-        $self->meta({});
         return;
     }
 
-    my $yaml;
-
-    # YAML::LoadFile dies when it cannot properly parse a file - catch it in
-    # an eval, and if it dies, return -again- just an empty hashref.
-    eval { $yaml = YAML::LoadFile($file); };
-    if ($@) {
-        print "Error parsing $file - Ignoring it.\n";
-        print "Please notify module upstream maintainer.\n";
-        $yaml = {};
+    my $meta = $self->main_file('META.json');
+    if ( -e $meta ) {
+        print "Using META.json\n" if $self->cfg->verbose;
+    }
+    else {
+        $meta = $self->main_file('META.yml');
+        if ( -e $meta ) {
+            print "Using META.yml\n" if $self->cfg->verbose;
+        }
+        else {
+            print "WARNING: Neither META.json nor META.yml was found\n";
+            return;
+        }
     }
 
-    if (ref $yaml ne 'HASH') {
-        print "$file does not contain a hash - Ignoring it\n";
-        $yaml = {};
-    }
-
-    # Returns a simple hashref with all the keys/values defined in META.yml
-    $self->meta($yaml);
+    $meta = CPAN::Meta->load_file($meta);
+    $self->meta( $meta->as_struct );
 }
 
 sub set_package_name {
     my $self = shift;
 
-    my $pkgname = lc $self->perlname;
-    $pkgname = 'lib' . $pkgname unless $pkgname =~ /^lib/;
-    $pkgname .= '-perl';
-
-    # ensure policy compliant names and versions (from Joeyh)...
-    $pkgname =~ s/[^-.+a-zA-Z0-9]+/-/g;
+    my $pkgname;
+    if (defined $self->cfg->packagename) {
+      $pkgname = $self->cfg->packagename;
+    }
+    else {
+        $pkgname = Debian::Control::FromCPAN->module_name_to_pkg_name( $self->perlname );
+    }
 
     $self->control->source->Source($pkgname)
         unless $self->control->source->Source;
 
-    $self->control->binary->Push( $pkgname =>
+    $self->control->binary_tie->Push( $pkgname =>
             Debian::Control::Stanza::Binary->new( { Package => $pkgname } ) )
-        unless $self->control->binary->FETCH($pkgname);
+        unless $self->control->binary->{$pkgname};
 }
 
 sub pkgname {
@@ -215,7 +221,7 @@ sub pkgname {
 
     my $self = shift;
 
-    my $pkg = $self->control->binary->Values(0)->Package;
+    my $pkg = $self->control->binary_tie->Values(0)->Package;
 
     defined($pkg) and $pkg ne ''
         or confess "called before set_package_name()";
@@ -253,13 +259,23 @@ sub extract_basic {
     $self->extract_name_ver();
 
     my $src = $self->control->source;
-    my $bin = $self->control->binary->Values(0);
+    my $bin = $self->control->binary_tie->Values(0);
 
     $src->Section('perl') unless defined $src->Section;
     $src->Priority('optional') unless defined $src->Priority;
 
-    $bin->Architecture('all');
-    find( sub { $self->check_for_xs }, $self->main_dir );
+    if ( $self->cfg->arch ) {
+        printf "Forcing architecture to '%s'\n", $self->cfg->arch;
+        $bin->Architecture( $self->cfg->arch );
+    }
+    else {
+        $bin->Architecture('all');
+        find( sub { $self->check_for_xs }, $self->main_dir );
+    }
+
+    $self->cfg->dh('9')
+        if $bin->Architecture eq 'any'
+        and not $self->cfg->_explicitly_set->{dh};
 
     printf(
         "Found: %s %s (%s arch=%s)\n",
@@ -269,20 +285,36 @@ sub extract_basic {
     $self->debian_dir( $self->main_file('debian') );
 
     find(
-        sub {
-            return if $File::Find::name =~ $self->cfg->exclude;
+        {   no_chdir => 1,
+            wanted   => sub {
+                return if $File::Find::name =~ $self->cfg->exclude;
 
-            if (/\.(pm|pod)$/) {
-                $self->extract_desc($_)
-                    unless $bin->short_description and $bin->long_description;
-                $self->extract_basic_copyright($_)
-                    unless $self->author and $self->copyright;
-            }
+                if (/\.(pm|pod)$/) {
+                    $self->extract_desc($_)
+                        unless $bin->short_description
+                            and $bin->long_description;
+                    $self->extract_basic_copyright($_)
+                        unless $self->author and $self->copyright;
+                }
+            },
         },
         $self->main_dir
     );
 }
 
+sub sanitize_version {
+    my $self = shift;
+    my ($ver) = @_;
+
+    return undef unless defined($ver);
+
+    $ver =~ s/^v//;
+    $ver =~ s/[^-.+a-zA-Z0-9]+/-/g;
+    $ver = "0$ver" unless $ver =~ /^\d/;
+
+    return $ver;
+}
+
 sub extract_name_ver {
     my ($self) = @_;
 
@@ -291,22 +323,51 @@ sub extract_name_ver {
     if ( defined $self->meta->{name} and defined $self->meta->{version} ) {
         $name = $self->meta->{name};
         $ver  = $self->meta->{version};
-        if ( $ver =~ s/^v// ) {    # v4.43.43?
-            $ver =~ s/\.(\d\d\d)(\d\d\d)/.$1.$2/;    # 2.003004 -> 2.003.004
-            $ver =~ s/\.0+/./g;                      # 2.003.004 -> 2.3.4
-        }
+    }
+    elsif ( defined $self->cfg->packagename and defined $self->cfg->version ) {
+        $name = $self->cfg->packagename;
+        $ver  = $self->cfg->version;
     }
     else {
-        $self->extract_name_ver_from_makefile( $self->makefile_pl );
+        if ( -e $self->build_pl ) {
+            print "Extracting name and version from Build.PL\n";
+            $self->extract_name_ver_from_build( $self->build_pl );
+        }
+        elsif ( -e $self->makefile_pl ) {
+            print "Extracting name and version from Makefile.PL\n";
+            $self->extract_name_ver_from_makefile( $self->makefile_pl );
+        }
+        else {
+            if ( $self->cfg->cpan ) {
+                my $name = $self->cfg->cpan;
+                $name =~ s/::/-/g;
+                $self->perlname($name);
+            }
+            else {
+                die "Unable to determine dist name, no Build.PL, no Makefile.PL\nPlease use --cpan.\n";
+            }
+        }
         $name = $self->perlname;
         $ver  = $self->version;
     }
 
-    # final sanitazing of name and version
-    $ver =~ s/[^-.+a-zA-Z0-9]+/-/g;
-    $ver = "0$ver" unless $ver =~ /^\d/;
+    $ver = $self->cfg->version
+        if $self->cfg->version;
 
+    defined($ver) and $ver ne ''
+        or die "Unable to determine dist version\. Please use --version.\n";
+
+    # final sanitazing of name and version
     $name =~ s/::/-/g;
+    $ver = $self->sanitize_version($ver) unless $self->cfg->version;
+
+    $name
+        or $ver
+        or die
+        "Unable to determine distribution name and version. Aborting.\n";
+
+    $name or die "Unable to determine distribution name. Aborting.\n";
+    $ver  or die "Unable to determine distribution version. Aborting.\n";
 
     $self->perlname($name);
     $self->version($ver);
@@ -314,13 +375,13 @@ sub extract_name_ver {
     $self->set_package_name;
 }
 
-sub extract_name_ver_from_makefile {
-    my ( $self, $makefile ) = @_;
+sub extract_name_ver_from_build {
+    my ( $self, $build ) = @_;
     my ( $file, $name, $ver, $vfrom, $dir );
 
     {
         local $/ = undef;
-        my $fh = $self->_file_r($makefile);
+        my $fh = $self->_file_r($build);
         $file = $fh->getline;
     }
 
@@ -329,46 +390,31 @@ sub extract_name_ver_from_makefile {
 
     # Get the name
     if ($file =~ /([\'\"]?)
-	    DISTNAME\1\s*
+	    dist_name\1\s*
 	    (=>|,)
 	    \s*
 	    ([\'\"]?)
 	    (\S+)\3/xs
         )
     {
-
-        # Regular MakeMaker
         $name = $4;
     }
     elsif (
         $file =~ /([\'\"]?)
-		 NAME\1\s*
+		 module_name\1\s*
 		 (=>|,)
 		 \s*
-		 ([\'\"]?)
-		 (\S+)\3/xs
+		 (\S+)/xs
         )
     {
+        $name = $self->unquote($2);
+        $name =~ s/::/-/g;
 
-        # Regular MakeMaker
-        $name = $4;
-    }
-    elsif (
-        $file =~ m{
-                        name
-                         \s*
-                         \(?                    # Optional open paren
-                             ([\'\"]?)          # Optional open quote
-                                 (\S+)          # Quoted name
-                             \1                 # Optional close quote
-                         \)?                    # Optional close paren
-                         \s*;
-                 }xs
-        )
-    {
-
-        # Module::Install syntax
-        $name = $2;
+        # just in case we need it later
+        $vfrom = $name;
+        $vfrom =~ s/-/::/g;
+        $vfrom =~s{::}{/}g;
+        $vfrom = "lib/$vfrom.pm";
     }
     $name =~ s/,.*$//;
 
@@ -384,7 +430,141 @@ sub extract_name_ver_from_makefile {
         $ver = $self->cfg->version;
 
     }
-    elsif ( $file =~ /([\'\"]?)VERSION\1\s*(=>|,)\s*([\'\"]?)(\S+)\3/s ) {
+    elsif ( $file =~ /([\'\"]?)\sdist_version\1\s*(=>|,)\s*([\'\"]?)([^\s,]*)\3/s ) {
+        $ver = $4;
+
+        # Where is the version taken from?
+        $vfrom = $4
+            if $file
+                =~ /([\'\"]?)dist_version_from\1\s*(=>|,)\s*([\'\"]?)([^\s,]*)\3/s;
+
+    }
+    elsif ( $file =~ /([\'\"]?)dist_version_from\1\s*(=>|,)\s*([\'\"]?)([^\s,]*)\3/s )
+    {
+        $vfrom = $4;
+
+    }
+
+    $dir = dirname($build) || './';
+
+    for ( ( $name, $ver ) ) {
+        next unless defined;
+        next unless /^\$/;
+
+        # decode simple vars
+        s/(\$\w+).*/$1/;
+        if ( $file =~ /\Q$_\E\s*=\s*([\'\"]?)(\S+)\1\s*;/ ) {
+            $_ = $2;
+        }
+    }
+
+    unless ( defined $ver ) {
+        local $/ = "\n";
+
+        # apply the method used by makemaker
+        if (    defined $dir
+            and defined $vfrom
+            and -f "$dir/$vfrom"
+            and -r "$dir/$vfrom" )
+        {
+            my $fh = $self->_file_r("$dir/$vfrom");
+            while ( my $lin = $fh->getline ) {
+                if ( $lin =~ /([\$*])(([\w\:\']*)\bVERSION)\b.*\=/ ) {
+                    no strict;
+
+                    #warn "ver: $lin";
+                    $ver = ( eval $lin )[0];
+                    last;
+                }
+            }
+            $fh->close;
+        }
+        else {
+            if ( $self->mod_cpan_version ) {
+                $ver = $self->mod_cpan_version;
+                warn "Cannot use internal module data to gather the "
+                    . "version; using cpan_version\n";
+            }
+            else {
+                die "Cannot use internal module data to gather the "
+                    . "version; use --cpan or --version\n";
+            }
+        }
+    }
+
+    $self->perlname($name);
+    $self->version($ver);
+
+    $self->set_package_name;
+
+    if ( defined($vfrom) ) {
+        $self->extract_desc("$dir/$vfrom");
+        $self->extract_basic_copyright("$dir/$vfrom");
+    }
+}
+
+sub extract_name_ver_from_makefile {
+    my ( $self, $makefile ) = @_;
+    my ( $file, $name, $ver, $vfrom, $dir );
+
+    {
+        local $/ = undef;
+        my $fh = $self->_file_r($makefile);
+        $file = $fh->getline;
+    }
+
+    # Get the name
+    if ($file =~ /([\'\"]?)
+	    DISTNAME\1\s*
+	    (=>|,)
+	    \s*
+	    (\S+)/xs
+        )
+    {
+
+        # Regular MakeMaker
+        $name = $self->unquote($3);
+    }
+    elsif (
+        $file =~ /([\'\"]?)
+		 NAME\1\s*
+		 (=>|,)
+		 \s*
+		 (\S+)\s*,?/xs
+        )
+    {
+
+        # Regular MakeMaker
+        $name = $self->unquote($3);
+    }
+    elsif (
+        $file =~ m{
+                        name
+                         \s*
+                                 (\S+)          # Quoted name
+                         \s*;
+                 }xs
+        )
+    {
+
+        # Module::Install syntax
+        $name = $self->unquote($1);
+    }
+    $name =~ s/,.*$//;
+
+    # band aid: need to find a solution also for build in directories
+    # warn "name is $name (cpan name: $self->cfg->cpan)\n";
+    $name = $self->cfg->cpan     if ( $name eq '__PACKAGE__' && $self->cfg->cpan );
+    $name = $self->cfg->cpanplus if ( $name eq '__PACKAGE__' && $self->cfg->cpanplus );
+
+    # Get the version
+    if ( defined $self->cfg->version ) {
+
+        # Explicitly specified
+        $ver = $self->cfg->version;
+
+    }
+    elsif ( $file =~ /([\'\"]?)\bVERSION\1\s*(=>|,)\s*([\'\"]?)([^\s,]*)\3/s ) {
 
         # Regular MakeMaker
         $ver = $4;
@@ -392,25 +572,25 @@ sub extract_name_ver_from_makefile {
         # Where is the version taken from?
         $vfrom = $4
             if $file
-                =~ /([\'\"]?)VERSION_FROM\1\s*(=>|,)\s*([\'\"]?)(\S+)\3/s;
+                =~ /([\'\"]?)VERSION_FROM\1\s*(=>|,)\s*([\'\"]?)([^\s,]*)\3/s;
 
     }
-    elsif ( $file =~ /([\'\"]?)VERSION_FROM\1\s*(=>|,)\s*([\'\"]?)(\S+)\3/s )
+    elsif ( $file =~ /([\'\"]?)VERSION_FROM\1\s*(=>|,)\s*([\'\"]?)([^\s,]*)\3/s )
     {
 
         # Regular MakeMaker pointing to where the version is taken from
         $vfrom = $4;
 
     }
-    elsif ( 
+    elsif (
         $file =~ m{
             \bversion\b\s*                  # The word version
             \(?\s*                          # Optional open-parens
             (['"]?)                         # Optional quotes
             ([\d_.]+)                       # The actual version.
             \1                              # Optional close-quotes
-            \s*\)?                          # Optional close-parens.
-        }sx 
+            \s*\)?                          # Optional close-parenthesis.
+        }sx
     ) {
 
         # Module::Install
@@ -478,7 +658,7 @@ sub extract_name_ver_from_makefile {
 sub extract_desc {
     my ( $self, $file ) = @_;
 
-    my $bin = $self->control->binary->Values(0);
+    my $bin = $self->control->binary_tie->Values(0);
     my $desc = $bin->short_description;
 
     $desc and return;
@@ -510,10 +690,16 @@ sub extract_desc {
         $desc = $my_desc;
     }
 
-    # Replace linefeeds (not followed by a space) in short description with
-    # spaces
-    $desc =~ s/\n(?=\S)/ /gs;
-    $desc =~ s/^\s+//;      # strip leading spaces
+    if ( defined($desc) ) {
+        # Replace linefeed (not followed by a space) in short description with
+        # spaces
+        $desc =~ s/\n(?=\S)/ /gs;
+        $desc =~ s/^\s+//;      # strip leading spaces
+    }
+
+    # have a fall-back for the short description
+    $desc ||= '(no short description found)';
+
     $bin->short_description($desc);
 
     my $long_desc;
@@ -524,6 +710,7 @@ sub extract_desc {
             || '';
         ( $modulename = $self->perlname ) =~ s/-/::/g;
         $long_desc =~ s/This module/$modulename/;
+        $long_desc =~ s/This library/$modulename/;
 
         local ($Text::Wrap::columns) = 78;
         $long_desc = fill( "", "", $long_desc );
@@ -550,12 +737,12 @@ sub check_for_xs {
     # we need the relative path here. Otherwise the check will give bogus
     # results if the working dir matches the pattern
     my $rel_path = substr( $File::Find::name, length( $self->main_dir ) );
-    ( $rel_path !~ m{/(?:examples?|samples|eg|t|docs?)/} )
+    ( $rel_path !~ m{/(?:examples?|samples|eg|t|docs|win32?|demos?)/} )
             and
     ( !$self->cfg->exclude or $rel_path !~ $self->cfg->exclude )
         && /\.(xs|c|cpp|cxx)$/i
         && do {
-        $self->control->binary->Values(0)->Architecture('any');
+        $self->control->binary_tie->Values(0)->Architecture('any');
         };
 }
 
@@ -623,7 +810,8 @@ sub extract_docs {
                     substr( $File::Find::name, length($dir) )
                     )
                     if (
-                        /^\b(README|TODO|BUGS|NEWS|ANNOUNCE)\b/i
+                        $File::Find::name ne $self->main_dir . '/README'
+                    and /^\b(README|TODO|BUGS|NEWS|ANNOUNCE)\b/i
                     and !/\.(pod|pm)$/
                     and ( !$self->cfg->exclude
                         or $File::Find::name !~ $self->cfg->exclude )
@@ -657,6 +845,16 @@ sub extract_examples {
     );
 }
 
+sub read_rules {
+    my $self = shift;
+
+    return if $self->rules;
+
+    my $file = $self->debian_file('rules');
+
+    $self->rules( Debian::Rules->new($file) );
+}
+
 sub create_rules {
     my ( $self ) = @_;
 
@@ -743,20 +941,20 @@ sub create_copyright {
     my ( $fh, %fields, @res, @incomplete, $year );
     $fh = $self->_file_w($filename);
 
-    # In case author string pawns more than one line, indent them all.
+    # In case author string spawns more than one line, indent them all.
     my $cprt_author = $self->author || '(information incomplete)';
     $cprt_author =~ s/\n/\n    /gs;
     $cprt_author =~ s/^\s*$/    ./gm;
 
-    push @res, "Format-Specification: http://svn.debian.org/wsvn/dep/web/deps/dep5.mdwn?op=file&rev=135";
+    push @res, 'Format: http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/';
 
     # Header section
     %fields = (
-        Name       => $self->perlname,
-        Maintainer => $cprt_author,
-        Source     => $self->upsurl
+        'Upstream-Name'    => $self->perlname,
+        'Upstream-Contact' => $cprt_author,
+        'Source'           => $self->upsurl
     );
-    for my $key ( keys %fields ) {
+    for my $key ( sort keys %fields ) {
         my $full = "$key";
         if ( $fields{$key} ) {
             push @res, "$full: $fields{$key}";
@@ -795,46 +993,56 @@ sub create_copyright {
             " This program is free software; you can redistribute it and/or modify\n"
             . " it under the terms of the Artistic License, which comes with Perl.\n"
             . " .\n"
-            . " On Debian GNU/Linux systems, the complete text of the Artistic License\n"
-            . " can be found in `/usr/share/common-licenses/Artistic'",
+            . " On Debian systems, the complete text of the Artistic License can be\n"
+            . " found in `/usr/share/common-licenses/Artistic'.",
+        'Artistic-2.0' => do {
+            require Software::License::Artistic_2_0;
+            my $artistic2 = Software::License::Artistic_2_0->new(
+                { holder => 'noname', } );
+            my $text = $artistic2->license;
+            $text =~ s/\n$//s;
+            $text =~ s/^\n/.\n/mg;
+            $text =~ s/^/ /mg;
+            $text;
+            },
         'GPL-1+' =>
             " This program is free software; you can redistribute it and/or modify\n"
             . " it under the terms of the GNU General Public License as published by\n"
             . " the Free Software Foundation; either version 1, or (at your option)\n"
             . " any later version.\n"
             . " .\n"
-            . " On Debian GNU/Linux systems, the complete text of the GNU General\n"
-            . " Public License can be found in `/usr/share/common-licenses/GPL'",
+            . " On Debian systems, the complete text of version 1 of the GNU General\n"
+            . " Public License can be found in `/usr/share/common-licenses/GPL-1'.",
         'GPL-2' =>
             " This program is free software; you can redistribute it and/or modify\n"
             . " it under the terms of the GNU General Public License as published by\n"
             . " the Free Software Foundation; version 2 dated June, 1991.\n"
             . " .\n"
-            . " On Debian GNU/Linux systems, the complete text of version 2 of the GNU\n"
-            . " General Public License can be found in `/usr/share/common-licenses/GPL-2'",
+            . " On Debian systems, the complete text of version 2 of the GNU General\n"
+            . " Public License can be found in `/usr/share/common-licenses/GPL-2'.",
         'GPL-2+' =>
             " This program is free software; you can redistribute it and/or modify\n"
             . " it under the terms of the GNU General Public License as published by\n"
             . " the Free Software Foundation; version 2 dated June, 1991, or (at your\n"
             . " option) any later version.\n"
             . " .\n"
-            . " On Debian GNU/Linux systems, the complete text of version 2 of the GNU\n"
-            . " General Public License can be found in `/usr/share/common-licenses/GPL-2'",
+            . " On Debian systems, the complete text of version 2 of the GNU General\n"
+            . " Public License can be found in `/usr/share/common-licenses/GPL-2'.",
         'GPL-3' =>
             " This program is free software; you can redistribute it and/or modify\n"
             . " it under the terms of the GNU General Public License as published by\n"
             . " the Free Software Foundation; version 3 dated June, 2007.\n"
             . " .\n"
-            . " On Debian GNU/Linux systems, the complete text of version 3 of the GNU\n"
-            . " General Public License can be found in `/usr/share/common-licenses/GPL-3'",
+            . " On Debian systems, the complete text of version 3 of the GNU General\n"
+            . " Public License can be found in `/usr/share/common-licenses/GPL-3'.",
         'GPL-3+' =>
             " This program is free software; you can redistribute it and/or modify\n"
             . " it under the terms of the GNU General Public License as published by\n"
             . " the Free Software Foundation; version 3 dated June, 2007, or (at your\n"
-            . " option) any later version\n"
+            . " option) any later version.\n"
             . " .\n"
-            . " On Debian GNU/Linux systems, the complete text of version 3 of the GNU\n"
-            . " General Public License can be found in `/usr/share/common-licenses/GPL-3'",
+            . " On Debian systems, the complete text of version 3 of the GNU General\n"
+            . " Public License can be found in `/usr/share/common-licenses/GPL-3'.",
         'Apache-2.0' =>
             " Licensed under the Apache License, Version 2.0 (the \"License\");\n"
             . " you may not use this file except in compliance with the License.\n"
@@ -846,8 +1054,8 @@ sub create_copyright {
             . " See the License for the specific language governing permissions and\n"
             . " limitations under the License.\n"
             . " .\n"
-            . " On Debian GNU/Linux systems, the complete text of the Apache License,\n"
-            . " Version 2.0 can be found in `/usr/share/common-licenses/Apache-2.0'",
+            . " On Debian systems, the complete text of the Apache License,\n"
+            . " Version 2.0 can be found in `/usr/share/common-licenses/Apache-2.0'.",
         'unparsable' =>
             " No known license could be automatically determined for this module.\n"
             . " If this module conforms to a commonly used license, please report this\n"
@@ -867,13 +1075,29 @@ sub create_copyright {
         # mind that many licenses are not meant to be used as
         # templates (i.e. you must add the author name and some
         # information within the licensing text as such).
-        if (   $self->meta->{license} and $self->meta->{license} =~ /perl/i
-            or $mangle_cprt =~ /terms\s*as\s*Perl\s*itself/is )
-        {
-            $licenses{'GPL-1+'}   = 1;
-            $licenses{'Artistic'} = 1;
+        if ( $self->meta->{license} ) {
+            foreach ( @{ $self->meta->{license} } ) {
+                if (/apache_2_0/) { $licenses{'Apache-2.0'}   = 1; next; }
+                if (/artistic_1/) { $licenses{'Artistic'}     = 1; next; }
+                if (/artistic_2/) { $licenses{'Artistic-2.0'} = 1; next; }
+
+                # EU::MM and M::B converts the 'gpl' without a version to gpl_1.
+                # As GPL without a version means *any* GPL, I think it's safe to use GPL-1+ here
+                if (/gpl_1/) { $licenses{'GPL-1+'} = 1; next; }
+
+                if (/perl_5/) {
+                    $licenses{'GPL-1+'}   = 1;
+                    $licenses{'Artistic'} = 1;
+                    next;
+                }
+            }
         }
         else {
+            if ( $mangle_cprt =~ /terms\s*as\s*Perl\s*itself/is ) {
+              $licenses{'GPL-1+'}   = 1;
+              $licenses{'Artistic'} = 1;
+            }
+
             if ( $mangle_cprt =~ /[^L]GPL/ ) {
                 if ( $mangle_cprt =~ /GPL.*version\s*1.*later\s+version/is ) {
                     $licenses{'GPL-1+'} = 1;
@@ -896,7 +1120,10 @@ sub create_copyright {
                 }
             }
 
-            if ( $mangle_cprt =~ /Artistic\s*License/is ) {
+            if ( $mangle_cprt =~ /Artistic\s*License\s*2\.0/is ) {
+                $licenses{'Artistic-2.0'} = 1;
+            }
+            elsif ( $mangle_cprt =~ /Artistic\s*License/is ) {
                 $licenses{'Artistic'} = 1;
             }
 
@@ -914,7 +1141,7 @@ sub create_copyright {
             }
         }
 
-        push @res, "License: " . join( ' or ', keys %licenses );
+        push @res, "License: " . join( ' or ', sort keys %licenses );
 
     }
     else {
@@ -923,7 +1150,7 @@ sub create_copyright {
     }
 
     # debian/* files information - We default to the module being
-    # licensed as the superset of the module and Perl itself.
+    # licensed as the super-set of the module and Perl itself.
     $licenses{'Artistic'} = $licenses{'GPL-1+'} = 1;
     $year = (localtime)[5] + 1900;
     push( @res, "", "Files: debian/*" );
@@ -936,10 +1163,10 @@ sub create_copyright {
     else {
         push @res, "Copyright: $year, " . $self->get_developer;
     }
-    push @res, "License: " . join( ' or ', keys %licenses );
+    push @res, "License: " . join( ' or ', sort keys %licenses );
 
     map { $texts{$_} && push( @res, '', "License: $_", $texts{$_} ) }
-        keys %licenses;
+        sort keys %licenses;
 
     $fh->print( join( "\n", @res, '' ) );
     $fh->close;
@@ -950,20 +1177,63 @@ sub create_copyright {
 
 sub upsurl {
     my $self = shift;
-    return sprintf( "http://search.cpan.org/dist/%s/", $self->perlname );
+    return sprintf( "https://metacpan.org/release/%s", $self->perlname );
 }
 
+
+my $ACTUAL_NAME_RE = '\pL[\s\pL\-\'\.]*\pL';
+
+# See http://www.faqs.org/rfcs/rfc2822.html
+# Section 3.4.1
+use Email::Address;
+my $EMAIL_RE = $Email::Address::addr_spec;
+
+my $EMAIL_CHANGES_RE = qr{
+    ^                           # beginining of line
+    \s+\*\s                     # item marker
+    Email\schange:\s            # email change token
+    ($ACTUAL_NAME_RE)           # actual name
+    \s+->\s+                    # gap between name and email
+    ($EMAIL_RE)                 # email address
+    $                           # end of line
+}xms;
+
+my $PERSON_PARSE_RE = qr{
+    \A                          # beginining of string
+    ($ACTUAL_NAME_RE)           # actual name
+    \s                          # gap
+    \<$EMAIL_RE\>               # logged email
+    \z                          # end of string
+}xms;
+
+# This is what needs fixing.
 sub copyright_from_changelog {
     my ( $self, $firstmaint, $firstyear ) = @_;
     my %maintainers = ();
     @{ $maintainers{$firstmaint} } = ($firstyear);
     my $chglog = Parse::DebianChangelog->init(
         { infile => $self->debian_file('changelog') } );
+    my %email_changes = ();
     foreach ( $chglog->data() ) {
         my $person      = $_->Maintainer;
         my $date        = $_->Date;
         my @date_pieces = split( " ", $date );
         my $year        = $date_pieces[3];
+        if (my %changes = ($_->Changes =~ m/$EMAIL_CHANGES_RE/xmsg)) {
+            # This way round since we are going backward in time thru changelog
+            foreach my $p (keys %changes) {
+                $changes{$p} =~ s{[\s\n]+$}{}xms;
+            }
+            %email_changes = (
+                %changes,
+                %email_changes
+            );
+        }
+        if (my ($name) = ($person =~ $PERSON_PARSE_RE)) {
+            if (exists $email_changes{$name}) {
+                $person = "$name <$email_changes{$name}>";
+            }
+        }
         if ( defined( $maintainers{$person} ) ) {
             push @{ $maintainers{$person} }, $year;
             @{ $maintainers{$person} } = sort( @{ $maintainers{$person} } );
@@ -1041,7 +1311,7 @@ sub module_build {
 Adds the list of dependencies to I<$dependencies> and shows I<$reason> if in
 verbose mode.
 
-Used to both bump a dependency ant tell the user why.
+Used to both bump a dependency and tell the user why.
 
 I<$dependencies> is an instance of L<Debian::Dependencies> class, and
 I<@dependencies> is a list of L<Debian::Dependency> instances or strings.
@@ -1070,18 +1340,22 @@ sub configure_cpan {
 
     return if $CPAN::Config_loaded;
 
-    CPAN::Config->load( be_silent => not $self->cfg->verbose );
+    my $save_cwd = getcwd();
+
+    CPAN::HandleConfig->load( be_silent => not $self->cfg->verbose )
+        if $self->cfg->network;
 
     unshift( @{ $CPAN::Config->{'urllist'} }, $self->cfg->cpan_mirror )
         if $self->cfg->cpan_mirror;
 
-    $CPAN::Config->{'build_dir'}         = $ENV{'HOME'} . "/.cpan/build";
-    $CPAN::Config->{'cpan_home'}         = $ENV{'HOME'} . "/.cpan/";
-    $CPAN::Config->{'histfile'}          = $ENV{'HOME'} . "/.cpan/history";
-    $CPAN::Config->{'keep_source_where'} = $ENV{'HOME'} . "/.cpan/source";
     $CPAN::Config->{'tar_verbosity'}     = $self->cfg->verbose ? 'v' : '';
     $CPAN::Config->{'load_module_verbosity'}
         = $self->cfg->verbose ? 'verbose' : 'silent';
+
+    $CPAN::Config->{build_requires_install_policy} = 'no';
+    $CPAN::Config->{prerequisites_policy} = 'ignore';
+
+    chdir $save_cwd;
 }
 
 =item discover_dependencies
@@ -1089,6 +1363,8 @@ sub configure_cpan {
 Just a wrapper around $self->control->discover_dependencies which provides the
 right parameters to it.
 
+Returns a list of missing modules.
+
 =cut
 
 sub discover_dependencies {
@@ -1096,7 +1372,8 @@ sub discover_dependencies {
 
     if ( my $apt_contents = $self->get_apt_contents ) {
 
-        my $wnpp_query
+        my $wnpp_query;
+        $wnpp_query
             = Debian::WNPP::Query->new(
             { cache_file => catfile( $self->cfg->home_dir, 'wnpp.cache' ) } )
             if $self->cfg->network;
@@ -1104,7 +1381,7 @@ sub discover_dependencies {
         # control->discover_dependencies needs configured CPAN
         $self->configure_cpan;
 
-        $self->control->discover_dependencies(
+        return $self->control->discover_dependencies(
             {   dir          => $self->main_dir,
                 verbose      => $self->cfg->verbose,
                 apt_contents => $self->apt_contents,
@@ -1116,9 +1393,11 @@ sub discover_dependencies {
     }
     else {
         warn "No APT contents can be loaded.\n";
-        warn "Please install 'apt-file' package and run 'apt-file update'\n";
-        warn "as root.\n";
+        warn "Please install 'apt-file' package (at least version 2.5.0) and\n";
+        warn "run 'apt-file update' as root.\n";
         warn "Dependencies not updated.\n";
+
+        return ();
     }
 }
 
@@ -1136,6 +1415,11 @@ The following special cases are detected:
 If L<Module::AutoInstall> is discovered in L<inc/>, debhelper dependency is
 raised to 7.2.13.
 
+=item Module::Build::Tiny
+
+if L<Module::Build::Tiny> is present in the build-dependencies, debhelper
+dependency is raised to 9.20130630.
+
 =item dh --with=quilt
 
 C<dh --with=quilt> needs debhelper 7.0.8 and quilt 0.46-7.
@@ -1144,12 +1428,20 @@ C<dh --with=quilt> needs debhelper 7.0.8 and quilt 0.46-7.
 
 C<dh --with=bash-completion> needs debhelper 7.0.8 and bash-completion 1:1.0-3.
 
+=item dh --with=perl_dbi
+
+C<dh --with=perl_dbi> needs debhelper 7.0.8 and libdbi-perl 1.612.
+
+=item dh --buildsystem=buildsystem
+
+C<dh --buildsystem=buildsystem> needs debhelper 7.3.7.
+
 =item quilt.make
 
 If F</usr/share/quilt/quilt.make> is included in F<debian/rules>, a
 build-dependency on C<quilt> is added.
 
-=item dhebhelper override targets
+=item debhelper override targets
 
 Targets named C<override_dh_...> are supported by debhelper since 7.0.50
 
@@ -1162,7 +1454,10 @@ L<http://bugs.debian.org/496157>) =back
 
 The proper build-dependency in this case is
 
-    perl (>= 5.10 ) | libmodule-build-perl
+    perl
+
+The on perl without a version is set as Lenny has already 5.10 which
+includes first Module::Build.
 
 =back
 
@@ -1177,9 +1472,11 @@ sub discover_utility_deps {
     $deps->remove( 'quilt', 'debhelper' );
 
     # start with the minimum
-    $deps->add( Debian::Dependency->new( 'debhelper', $self->cfg->dh ) );
+    my $debhelper_version = $self->cfg->dh;
+    $debhelper_version = '9.20120312' if $debhelper_version eq '9';
+    $deps->add( Debian::Dependency->new( 'debhelper', $debhelper_version ) );
 
-    if ( $control->binary->Values(0)->Architecture eq 'all' ) {
+    if ( $control->binary_tie->Values(0)->Architecture eq 'all' ) {
         $control->source->Build_Depends_Indep->add('perl');
     }
     else {
@@ -1189,6 +1486,9 @@ sub discover_utility_deps {
     $self->explained_dependency( 'Module::AutoInstall', $deps,
         'debhelper (>= 7.2.13)' )
         if -e catfile( $self->main_dir, qw( inc Module AutoInstall.pm ) );
+    $self->explained_dependency( 'Module::Build::Tiny', $deps,
+        'debhelper (>= 9.20130630)' )
+        if $deps->has('libmodule-build-tiny-perl');
 
     for ( @{ $self->rules->lines } ) {
         $self->explained_dependency( 'dh --with', $deps,
@@ -1201,11 +1501,17 @@ sub discover_utility_deps {
         ) if /dh\s+.*--with[= ]quilt/;
 
         $self->explained_dependency(
-            'dh -with=bash-completion',
+            'dh --with=bash-completion',
             $deps,
             'bash-completion (>= 1:1.0-3)'
         ) if (/dh\s+.*--with[= ]bash[-_]completion/);
 
+        $self->explained_dependency(
+            'dh --with=perl_dbi',
+            $deps,
+            'libdbi-perl (>= 1.612)'
+        ) if (/dh\s+.*--with[= ]perl[-_]dbi/);
+
         $self->explained_dependency( 'override_dh_* target',
             $deps, 'debhelper (>= 7.0.50)' )
             if /^override_dh_/;
@@ -1216,6 +1522,11 @@ sub discover_utility_deps {
         $self->explained_dependency( 'dh* --max-parallel',
             $deps, 'debhelper (>= 7.4.4)' )
             if /dh.* --max-parallel/;
+
+        # Modular --buildsystem support for debhelper needs 7.3.7.
+        $self->explained_dependency( 'dh* --buildsystem',
+            $deps, 'debhelper (>= 7.3.7)' )
+            if /dh.* --buildsystem/;
     }
 
     if (    -e $self->main_file('Makefile.PL')
@@ -1225,22 +1536,29 @@ sub discover_utility_deps {
             'Compatibility Makefile.PL',
             $deps,
             'debhelper (>= 7.0.17)',
-            'perl (>= 5.10) | libmodule-build-perl'
+            'perl'
         ) if $self->makefile_pl_is_MBC;
     }
 
     # there are old packages that still build-depend on libmodule-build-perl
+    # or perl (>= 5.10) | libmodule-build-perl.
     # Since M::B is part of perl 5.10, the build-dependency needs correction
+    # and we replace this Build-Depends with simply perl, as lenny has the
+    # required version.
+    # Remove perl from Build-Depends-Indep as then perl will be already in
+    # Build-Depends.
     if ( $self->module_build eq 'Module-Build' ) {
+        $deps->remove('perl (>= 5.10) | libmodule-build-perl');
         $deps->remove('libmodule-build-perl');
+        $control->source->Build_Depends_Indep->remove('perl');
         $self->explained_dependency( 'Module::Build', $deps,
-            'perl (>= 5.10) | libmodule-build-perl' );
+            'perl' );
     }
 
     # some mandatory dependencies
-    my $bin_deps = $control->binary->Values(0)->Depends;
+    my $bin_deps = $control->binary_tie->Values(0)->Depends;
     $bin_deps += '${shlibs:Depends}'
-        if $self->control->binary->Values(0)->Architecture eq 'any';
+        if $self->control->binary_tie->Values(0)->Architecture eq 'any';
     $bin_deps += '${misc:Depends}, ${perl:Depends}';
 }
 
@@ -1289,6 +1607,24 @@ sub backup_file {
     }
 }
 
+=item unquote(I<string>)
+
+Runs its argument through L<Text::Balanced>'s C<extract_quotelike> method and
+returns the extracted content with quotes removed. Dies if C<extract_quotelike>
+can't find quoted string.
+
+=cut
+
+sub unquote {
+    my ( $self, $input ) = @_;
+
+    my $unquoted = (extract_quotelike($input))[5];
+
+    die "Unable to find quoted string in [$input]" unless defined $unquoted;
+
+    return $unquoted;
+}
+
 =back
 
 =cut
@@ -1323,13 +1659,15 @@ sub _file_w {
 
 =item Copyright (C) 2006 Frank Lichtenheld <djpig@debian.org>
 
-=item Copyright (C) 2007-2010 Gregor Herrmann <gregoa@debian.org>
+=item Copyright (C) 2007-2013 Gregor Herrmann <gregoa@debian.org>
 
-=item Copyright (C) 2007-2010 Damyan Ivanov <dmn@debian.org>
+=item Copyright (C) 2007,2008,2009,2010,2012,2013 Damyan Ivanov <dmn@debian.org>
 
 =item Copyright (C) 2008, Roberto C. Sanchez <roberto@connexer.com>
 
-=item Copyright (C) 2009-2010, Salvatore Bonaccorso <salvatore.bonaccorso@gmail.com>
+=item Copyright (C) 2009-2011, Salvatore Bonaccorso <carnil@debian.org>
+
+=item Copyright (C) 2011, Nicholas Bamber <nicholas@periapt.co.uk>
 
 =back
 
