@@ -2,6 +2,7 @@ package DhMakePerl::Command::make;
 
 use warnings;
 use strict;
+our $VERSION = '0.77';
 use 5.010;    # we use smart matching
 
 use base 'DhMakePerl::Command::Packaging';
@@ -13,23 +14,16 @@ __PACKAGE__->mk_accessors(
         perlname version pkgversion
         copyright author
         extrasfields  extrapfields
-        mod_cpan_version
         docs examples
         )
 );
 
 =head1 NAME
 
-DhMakePerl - create Debian source package from CPAN dist
-
-=head1 VERSION
-
-Version 0.65
+DhMakePerl::Command::make - implementation of 'dh-make-perl make'
 
 =cut
 
-our $VERSION = '0.65';
-
 =head1 SYNOPSIS
 
 TO BE FILLED
@@ -50,12 +44,14 @@ use CPAN ();
 use Debian::Dependencies      ();
 use Debian::Dependency        ();
 use Debian::WNPP::Query;
-use DhMakePerl::Utils qw( find_cpan_module is_core_module );
+use DhMakePerl::Utils qw(
+    find_cpan_module find_cpan_distribution
+    is_core_module );
 use Email::Date::Format qw(email_date);
 use File::Basename qw( basename dirname );
 use File::Copy qw( copy move );
 use File::Path ();
-use File::Spec::Functions qw( catfile );
+use File::Spec::Functions qw( catdir catfile updir );
 use Module::Depends            ();
 use Module::Build::ModuleInfo;
 use Text::Wrap qw( wrap );
@@ -86,6 +82,8 @@ sub execute {
 
     $self->extract_basic();
 
+    $tarball //= $self->guess_tarball if $self->cfg->{vcs} eq 'git';
+
     unless ( defined $self->cfg->version ) {
         $self->pkgversion( $self->version . '-1' );
     }
@@ -95,18 +93,28 @@ sub execute {
 
     $self->fill_maintainer;
 
-    my $bin = $self->control->binary->Values(0);
+    my $bin = $self->control->binary_tie->Values(0);
     $bin->short_description( $self->cfg->desc )
         if $self->cfg->desc;
 
-    move(
-        $tarball,
-        sprintf(
-            "%s/%s_%s.orig.tar.gz",
-            dirname($tarball), $self->pkgname, $self->version
-        )
-    ) if ( $tarball && $tarball =~ /(?:\.tar\.gz|\.tgz)$/ );
+    if ( $tarball and $tarball =~ /(?:\.tar\.gz|\.tgz)$/ ) {
+        my $dest = sprintf( "%s/%s_%s.orig.tar.gz",
+            dirname($tarball), $self->pkgname, $self->version );
 
+        move( $tarball, $dest ) or die "move($tarball, $dest): $!";
+
+        $tarball = $dest;
+    }
+
+    # Here I init the git repo. If the upstream has a debian/ directory, this is
+    # removed in a separate git commit
+    $self->git_import_upstream__init_debian
+        if $self->cfg->{vcs} eq 'git';
+
+    # if the upstream has a debian/ directory, rename it to debian.bak so that
+    # dh-make-perl can create its own debian/ directory. If we're creating a git
+    # repo, the original debian/ directory was already dealt with by
+    # git_import_upstream__init_debian()
     if ( -d $self->debian_dir ) {
         $self->warning( $self->debian_dir . ' already exists' );
         my $bak = $self->debian_dir . '.bak';
@@ -162,17 +170,35 @@ sub execute {
 
     # now that rules are there, see if we need some dependency for them
     $self->discover_utility_deps( $self->control );
+    $self->control->prune_perl_deps;
+    $self->prune_deps;
     $src->Standards_Version( $self->debstdversion );
     $src->Homepage( $self->upsurl );
-    if ( $self->cfg->pkg_perl and my $cpan = $self->cfg->cpan ) {
-        $self->control->source->Vcs_Svn(
-            sprintf( "svn://svn.debian.org/pkg-perl/trunk/%s/",
-                $self->pkgname )
-        );
-        $self->control->source->Vcs_Browser(
-            sprintf( "http://svn.debian.org/viewsvn/pkg-perl/trunk/%s/",
-                $self->pkgname )
-        );
+    if ( $self->cfg->pkg_perl ) {
+        my $vcs = lc( $self->cfg->vcs );
+        if ( $vcs eq 'svn' ) {
+            $self->control->source->Vcs_Svn(
+                sprintf( "svn://svn.debian.org/pkg-perl/trunk/%s/",
+                    $self->pkgname )
+            );
+            $self->control->source->Vcs_Browser(
+                sprintf( "http://anonscm.debian.org/viewvc/pkg-perl/trunk/%s/",
+                    $self->pkgname )
+            );
+        }
+        elsif ( $vcs eq 'git' ) {
+            $self->control->source->Vcs_Git(
+                sprintf( "git://anonscm.debian.org/pkg-perl/packages/%s.git",
+                    $self->pkgname )
+            );
+            $self->control->source->Vcs_Browser(
+                sprintf( "http://anonscm.debian.org/gitweb/?p=pkg-perl/packages/%s.git",
+                    $self->pkgname )
+            );
+        }
+        else {
+            warn "Version control system '$vcs' not known. Please submit a patch :)\n";
+        }
     }
     $self->control->write( $self->debian_file('control') );
 
@@ -182,13 +208,8 @@ sub execute {
     #create_readme("$debiandir/README.Debian");
     $self->create_copyright( $self->debian_file('copyright') );
     $self->update_file_list( docs => $self->docs, examples => $self->examples );
-    $self->build_package
-        if $self->cfg->build or $self->cfg->install;
-    $self->install_package if $self->cfg->install;
-    print "--- Done\n" if $self->cfg->verbose;
 
-    $self->package_already_exists($apt_contents) 
-        or $self->modules_already_packaged($apt_contents);
+
 
     if ( $self->cfg->recursive ) {
         $already_done //= {};
@@ -213,42 +234,100 @@ sub execute {
         }
     }
 
+    $self->git_add_debian($tarball)
+        if $self->cfg->{vcs} eq 'git';
+
+    $self->build_source_package
+        if $self->cfg->build_source;
+    $self->build_package
+        if $self->cfg->build or $self->cfg->install;
+    $self->install_package if $self->cfg->install;
+    print "--- Done\n" if $self->cfg->verbose;
+
+    $self->package_already_exists($apt_contents)
+        or $self->modules_already_packaged($apt_contents);
+
+    # explicitly call Debian::Rules destroy
+    # this is needed because after the rename the object's
+    # destroy method would update a file on a stale path
+    $self->rules( undef );
+    $self->rename_to_debian_package_dir;
+
     return(0);
 }
 
+sub guess_tarball {
+    my $self = shift;
+
+    my $try = catfile( $self->main_dir, '..',
+              $self->control->source->Source . '_'
+            . $self->version
+            . '.orig.tar.gz' );
+
+    print "Trying $try...";
+    if ( -f $try ) {
+        print " found!\n";
+        return $try;
+    }
+    else {
+        print " not found.\n";
+        return undef;
+    }
+}
+
 sub setup_dir {
     my ($self) = @_;
 
-    my ( $dist, $mod, $tarball );
+    my ( $tarball );
     if ( $self->cfg->cpan ) {
-        my ($new_maindir, $orig_pwd);
+        my ( $new_maindir, $orig_pwd, $mod, $dist );
 
         # CPAN::Distribution::get() sets $ENV{'PWD'} to $CPAN::Config->{build_dir}
         # so we have to save it here
         $orig_pwd = $ENV{'PWD'};
 
         # Is the module a core module?
-        if ( is_core_module( $self->cfg->cpan ) ) {
-            die $self->cfg->cpan 
+       if ( is_core_module( $self->cfg->cpan ) ) {
+            die $self->cfg->cpan
             . " is a standard module. Will not build without --core-ok.\n"
                 unless $self->cfg->core_ok;
         }
 
         $self->configure_cpan;
 
-        $mod = find_cpan_module( $self->cfg->cpan )
-            or die "Can't find '" . $self->cfg->cpan . "' module on CPAN\n";
-        $self->mod_cpan_version( $mod->cpan_version );
+        if ( $mod = find_cpan_module( $self->cfg->cpan ) ) {
+            $self->mod_cpan_version( $mod->cpan_version );
 
-        $tarball = $CPAN::Config->{'keep_source_where'} . "/authors/id/";
+            $dist = $CPAN::META->instance( 'CPAN::Distribution',
+                $mod->cpan_file );
+        }
+        elsif ( $dist = find_cpan_distribution( $self->cfg->cpan ) ) {
+            my $ver;
+            if ( $dist->base_id =~ /-v?(\d[\d._]*)\./ ) {
+                $self->mod_cpan_version($1);
+            }
+            else {
+                die "Unable to determine the version of "
+                    . $dist->base_id . "\n";
+            }
+        }
+        else {
+            die "Can't find '"
+                . $self->cfg->cpan
+                . "' module or distribution on CPAN\n";
+        }
 
-        $dist = $CPAN::META->instance( 'CPAN::Distribution',
-            $mod->cpan_file );
-        $dist->get || die "Cannot get ", $mod->cpan_file, "\n"; # <- here $ENV{'PWD'} gets set to $HOME/.cpan/build
-        $tarball .= $mod->cpan_file;
+        $dist->get;     # <- here $ENV{'PWD'} gets set to $HOME/.cpan/build
+        chdir $orig_pwd;   # so set it back
+        $dist->pretty_id =~ /^(.)(.)/;
+        $tarball = $CPAN::Config->{'keep_source_where'} . "/authors/id/$1/$1$2/";
+        # the file is under authors/id/A/AU/AUTHOR directory
+        # how silly there is no $dist->filename method
+
+        $tarball .= $dist->pretty_id;
         $self->main_dir( $dist->dir );
 
-        copy( $tarball, $orig_pwd );
+        copy( $tarball, $orig_pwd ) or die "copy($tarball, $orig_pwd): $!";
         $tarball = $orig_pwd . "/" . basename($tarball);
 
         # build_dir contains a random part since 1.88_59
@@ -257,7 +336,7 @@ sub setup_dir {
 
         # rename existing directory
         if ( -d $new_maindir
-            && system( "mv", "$new_maindir", "$new_maindir.$$" ) == 0 )
+            && rename $new_maindir, "$new_maindir.$$" )
         {
             print '=' x 70, "\n";
             print
@@ -272,7 +351,7 @@ sub setup_dir {
     elsif ( $self->cfg->cpanplus ) {
         die "CPANPLUS support is b0rken at the moment.";
 
-        #  	        my ($cb, $href, $file);
+# 		my ($cb, $href, $file);
 
 # 		eval "use CPANPLUS 0.045;";
 # 		$cb = CPANPLUS::Backend->new(conf => {debug => 1, verbose => 1});
@@ -288,6 +367,17 @@ sub setup_dir {
         my $maindir = shift(@ARGV) || '.';
         $maindir =~ s/\/$//;
         $self->main_dir($maindir);
+        my $guessed_tarball = catfile( $self->start_dir, "..",
+            basename( $self->start_dir ) . ".tar.gz" );
+
+        print "Trying $guessed_tarball...";
+        if ( -f $guessed_tarball ) {
+            $tarball = $guessed_tarball;
+            print " found.\n";
+        }
+        else {
+            print " not found.\n";
+        }
     }
     return $tarball;
 }
@@ -299,23 +389,36 @@ sub build_package {
     # uhmf! dpkg-genchanges doesn't cope with the deb being in another dir..
     #system("dpkg-buildpackage -b -us -uc " . $self->cfg->dbflags) == 0
     system("fakeroot make -C $main_dir -f debian/rules clean");
-    system("make -C $main_dir -f debian/rules build") == 0 
+    system("make -C $main_dir -f debian/rules build") == 0
         || die "Cannot create deb package: 'debian/rules build' failed.\n";
     system("fakeroot make -C $main_dir -f debian/rules binary") == 0
         || die "Cannot create deb package: 'fakeroot debian/rules binary' failed.\n";
 }
 
+sub build_source_package {
+    my ( $self ) = @_;
+
+    my $main_dir = $self->main_dir;
+    # uhmf! dpkg-genchanges doesn't cope with the deb being in another dir..
+    #system("dpkg-buildpackage -S -us -uc " . $self->cfg->dbflags) == 0
+    system("fakeroot make -C $main_dir -f debian/rules clean");
+    system("dpkg-source -b $main_dir") == 0
+        || die "Cannot create source package: 'dpkg-source -b' failed.\n";
+}
+
 sub install_package {
     my ($self) = @_;
 
     my ( $archspec, $debname );
 
-    if ( $self->arch eq 'any' ) {
+    my $arch = $self->control->binary_tie->Values(0)->Architecture;
+
+    if ( !defined $arch || $arch eq 'any' ) {
         $archspec = `dpkg --print-architecture`;
         chomp($archspec);
     }
     else {
-        $archspec = $self->arch;
+        $archspec = $arch;
     }
 
     $debname = sprintf( "%s_%s-1_%s.deb", $self->pkgname, $self->version,
@@ -423,13 +526,59 @@ sub create_watch {
 
     my $fh = $self->_file_w($filename);
 
-    my $version_re = 'v?(\d[\d.-]+)\.(?:tar(?:\.gz|\.bz2)?|tgz|zip)';
+    my $version_re = 'v?(\d[\d.-]*)\.(?:tar(?:\.gz|\.bz2)?|tgz|zip)';
 
     $fh->printf( "version=3\n%s   .*/%s-%s\$\n",
         $self->upsurl, $self->perlname, $version_re );
     $fh->close;
 }
 
+sub search_pkg_perl {
+    my $self = shift;
+
+    return undef unless $self->cfg->network;
+
+    my $pkg = $self->pkgname;
+
+    require LWP::UserAgent;
+    require LWP::ConnCache;
+
+    my ( $ua, $resp );
+
+    $ua = LWP::UserAgent->new;
+    $ua->env_proxy;
+    $ua->conn_cache( LWP::ConnCache->new );
+
+    $resp = $ua->get(
+        "http://anonscm.debian.org/gitweb/?p=pkg-perl/packages/$pkg.git");
+    return { url => $resp->request->uri }
+        if $resp->is_success;
+
+    $resp = $ua->get(
+        "http://anonscm.debian.org/gitweb/?p=pkg-perl/attic/$pkg.git");
+    return { url => $resp->request->uri }
+        if $resp->is_success;
+
+    return undef;
+}
+
+sub rename_to_debian_package_dir {
+    my( $self ) = @_;
+    return unless $self->cfg->cpan;
+
+    my $maindir = $self->main_dir;
+    my $newmaindir = catdir( $maindir, updir(), $self->pkgname );
+
+    if( -d $newmaindir ) {
+      warn "$newmaindir already exists, skipping rename";
+      return;
+    }
+
+    rename $maindir, $newmaindir or die "rename failed: $self->main_dir to $newmaindir";
+    $self->main_dir( $newmaindir );
+    return;
+}
+
 sub package_already_exists {
     my( $self, $apt_contents ) = @_;
 
@@ -453,8 +602,7 @@ sub package_already_exists {
         warn "Description: $short_desc\n";
     }
     elsif ($apt_contents) {
-        my $found
-            = $apt_contents->find_perl_module_package( $self->perlname );
+        $found = $apt_contents->find_perl_module_package( $self->perlname );
 
         if ($found) {
             ( my $mod_name = $self->perlname ) =~ s/-/::/g;
@@ -463,11 +611,19 @@ sub package_already_exists {
             warn "        already contains a module named "
                 . $self->perlname . "\n";
         }
+        elsif ( $found = $self->search_pkg_perl ) {
+            warn "********************\n";
+            warn sprintf(
+                "The Debian Perl Group has a repository for the %s package\n  at %s\n",
+                $self->pkgname, $found->{url} );
+            warn "You may want to contact them to avoid duplication of effort.\n";
+            warn "More information is available at https://wiki.debian.org/Teams/DebianPerlGroup\n";
+        }
     }
     else {
         ( my $mod_name = $self->perlname ) =~ s/-/::/g;
-        require Debian::DpkgList;
-        my @found = Debian::DpkgList->scan_perl_mod($mod_name);
+        require Debian::DpkgLists;
+        my @found = Debian::DpkgLists->scan_perl_mod($mod_name);
 
         if (@found) {
             warn "**********\n";
@@ -488,15 +644,21 @@ sub modules_already_packaged {
 
     File::Find::find(
         sub {
-            if ( basename($File::Find::dir)
-                =~ /^(?:\.(?:git|svn|hg|)|CVS|eg|samples?|examples?|t)$/ )
+            if (basename($File::Find::dir)
+                =~ /^(?:
+                    \.(?:git|svn|hg|)
+                    |CVS
+                    |eg|samples?|examples?
+                    |t|xt
+                    |inc|privinc
+                    )$/x
+                )
             {
                 $File::Find::prune = 1;
                 return;
             }
             if (/.+\.pm$/) {
-                my $mi = Module::Build::ModuleInfo->new_from_file(
-                    $File::Find::name);
+                my $mi = Module::Build::ModuleInfo->new_from_file($_);
                 push @modules, $mi->packages_inside;
             }
         },
@@ -544,6 +706,64 @@ EOF
     return $found ? 1 : 0;
 }
 
+sub git_import_upstream__init_debian {
+    my ( $self ) = @_;
+
+    require Git;
+
+    Git::command( 'init', $self->main_dir );
+
+    my $git = Git->repository( $self->main_dir );
+    $git->command( qw(symbolic-ref HEAD refs/heads/upstream) );
+    $git->command( 'add', '.' );
+    $git->command( 'commit', '-m',
+              "Import original source of "
+            . $self->perlname . ' '
+            . $self->version );
+    $git->command( 'tag', "upstream/".$self->version, 'upstream' );
+
+    $git->command( qw( checkout -b master upstream ) );
+    if ( -d $self->debian_dir ) {
+      # remove debian/ directory if the upstream ships it. This goes into the
+      # 'master' branch, so the 'upstream' branch contains the original debian/
+      # directory, and thus matches the pristine-tar. Here I also remove the
+      # debian/ directory from the working tree; git has the history, so I don't
+      # need the debian.bak
+      $git->command( 'rm', '-r', $self->debian_dir );
+      $git->command( 'commit', '-m',
+                     'Removed debian directory embedded in upstream source' );
+    }
+}
+
+sub git_add_debian {
+    my ( $self, $tarball ) = @_;
+
+    require Git;
+    require File::Which;
+
+    my $git = Git->repository( $self->main_dir );
+    $git->command( 'add', 'debian' );
+    $git->command( 'commit', '-m', 'Initial packaging by dh-make-perl' );
+    $git->command(
+        qw( remote add origin ),
+        sprintf( "ssh://git.debian.org/git/pkg-perl/packages/%s.git",
+            $self->pkgname ),
+    ) if $self->cfg->pkg_perl;
+
+    if ( File::Which::which('pristine-tar') and $tarball ) {
+        $ENV{GIT_DIR} = File::Spec->catdir( $self->main_dir, '.git' );
+        system( 'pristine-tar', 'commit', $tarball, "upstream/".$self->version ) >= 0
+            or warn "error running pristine-tar: $!\n";
+    }
+    else {
+        warn "W: pristine-tar not available. Please run\n";
+        warn "W:     apt-get install pristine-tar\n";
+        warn "W:  followed by\n";
+        warn "W:     pristine-tar commit $tarball upstream/"
+            . $self->version . "\n";
+    }
+}
+
 =item warning I<string> ...
 
 In verbose mode, prints supplied arguments on STDERR, prepended with C<W: > and
@@ -608,13 +828,15 @@ L<http://bugs.debian.org/dh-make-perl>
 
 =item Copyright (C) 2006 Frank Lichtenheld <djpig@debian.org>
 
-=item Copyright (C) 2007-2010 Gregor Herrmann <gregoa@debian.org>
+=item Copyright (C) 2007-2013 Gregor Herrmann <gregoa@debian.org>
 
-=item Copyright (C) 2007-2010 Damyan Ivanov <dmn@debian.org>
+=item Copyright (C) 2007,2008,2009,2010,2011,2012 Damyan Ivanov <dmn@debian.org>
 
 =item Copyright (C) 2008, Roberto C. Sanchez <roberto@connexer.com>
 
-=item Copyright (C) 2009-2010, Salvatore Bonaccorso <salvatore.bonaccorso@gmail.com>
+=item Copyright (C) 2009-2010, Salvatore Bonaccorso <carnil@debian.org>
+
+=item Copyright (C) 2013, Axel Beckert <abe@debian.org>
 
 =back
 
